import { MatcherException } from '../../exceptions/matcher.exception';
import { IRequestTime } from '../interfaces/time-request.interface';
import { TimingFrequency } from './timing';

export class Time {
  _timeRequest: IRequestTime;
  _defaultMarginDuration: number;
  _defaultValidityDuration: number;
  frequency: TimingFrequency;
  fromDate: Date;
  toDate: Date;
  schedule: Array<Date>;
  marginDurations: Array<number>;

  constructor(
    timeRequest: IRequestTime,
    defaultMarginDuration: number,
    defaultValidityDuration: number,
  ) {
    this._timeRequest = timeRequest;
    this._defaultMarginDuration = defaultMarginDuration;
    this._defaultValidityDuration = defaultValidityDuration;
  }

  init() {
    this._validateBaseDate();
    this._validatePunctualDate();
    this._validateRecurrentDate();
    console.log(this.fromDate);
  }

  _validateBaseDate() {
    if (!this._timeRequest.departure && !this._timeRequest.fromDate) {
      throw new MatcherException(3, 'departure or fromDate is required');
    }
  }

  _validatePunctualDate() {
    if (this._timeRequest.departure) {
      this.fromDate = new Date(this._timeRequest.departure);
      if (!this._isDate(this.fromDate)) {
        throw new MatcherException(3, 'Wrong departure date');
      }
    }
  }

  _validateRecurrentDate() {
    if (this._timeRequest.fromDate) {
      this.fromDate = new Date(this._timeRequest.fromDate);
      if (!this._isDate(this.fromDate)) {
        throw new MatcherException(3, 'Wrong fromDate');
      }
    }
  }

  _isDate(date: Date) {
    return date instanceof Date && isFinite(+date);
  }
}
