import { MatchRequest } from '../domain/dtos/match.request';
import { Geography } from '../domain/entities/geography';
import { Person } from '../domain/entities/person';
import { Requirement } from '../domain/entities/requirement';
import { Role } from '../domain/entities/role.enum';
import { Settings } from '../domain/entities/settings';
import { Time } from '../domain/entities/time';
import { IDefaultParams } from '../domain/interfaces/default-params.interface';

export class MatchQuery {
  private readonly _matchRequest: MatchRequest;
  private readonly _defaultParams: IDefaultParams;
  person: Person;
  exclusions: Array<number>;
  time: Time;
  geography: Geography;
  roles: Array<Role>;
  requirement: Requirement;
  settings: Settings;

  constructor(matchRequest: MatchRequest, defaultParams: IDefaultParams) {
    this._matchRequest = matchRequest;
    this._defaultParams = defaultParams;
    this._initialize();
    this._setPerson();
    this._setExclusions();
    this._setRoles();
    this._setTime();
    // console.log(this);
  }

  _initialize() {
    if (
      this._matchRequest.driver === undefined &&
      this._matchRequest.passenger === undefined
    )
      this._matchRequest.passenger = true;
    this.geography = new Geography();
    this.requirement = new Requirement();
    this.settings = new Settings();
  }

  _setPerson() {
    this.person = new Person(
      this._matchRequest,
      this._defaultParams.DEFAULT_IDENTIFIER,
      this._defaultParams.MARGIN_DURATION,
    );
    this.person.init();
  }

  _setExclusions() {
    this.exclusions = [];
    if (this._matchRequest.identifier)
      this.exclusions.push(this._matchRequest.identifier);
    if (this._matchRequest.exclusions)
      this.exclusions.push(...this._matchRequest.exclusions);
  }

  _setRoles() {
    this.roles = [];
    if (this._matchRequest.driver) this.roles.push(Role.DRIVER);
    if (this._matchRequest.passenger) this.roles.push(Role.PASSENGER);
  }

  _setTime() {
    this.time = new Time(
      this._matchRequest,
      this._defaultParams.MARGIN_DURATION,
      this._defaultParams.VALIDITY_DURATION,
    );
    this.time.init();
  }
}
